@@ -5,4 +5,4 @@ from __future__ import absolute_import
 from cbc_sdk.audit_remediation.base import (Run, RunHistory, Result, DeviceSummary,
                                     ResultFacet, DeviceSummaryFacet,
                                     ResultQuery, FacetQuery, RunQuery,
-                                    RunHistoryQuery, Template)
+                                    RunHistoryQuery, Template, TemplateHistory)
