@@ -528,10 +528,11 @@ class NewBaseModel(object, metaclass=CbMetaModel):
         """
         lines = []
         lines.append("{0:s} object, bound to {1:s}.".format(self.__class__.__name__, self._cb.session.server))
-        if self._last_refresh_time:
-            lines.append(" Last refreshed at {0:s}".format(time.ctime(self._last_refresh_time)))
-        if not self._full_init:
-            lines.append(" Partially initialized. Use .refresh() to load all attributes")
+        if not issubclass(type(self), UnrefreshableModel):
+            if self._last_refresh_time:
+                lines.append(" Last refreshed at {0:s}".format(time.ctime(self._last_refresh_time)))
+            if not self._full_init:
+                lines.append(" Partially initialized. Use .refresh() to load all attributes")
         lines.append("-" * 79)
         lines.append("")
 
@@ -816,61 +817,72 @@ class BaseQuery(object):
     def _clone(self):
         return self.__class__(self._query)
 
+    def _perform_query(self):
+        # This has the effect of generating an empty iterator.
+        yield from ()
+
+
+class IterableQueryMixin:
+    """A mix-in to provide iterability to a query."""
+
     def all(self):
         """
-        Returns the objects that this query has located, all at once.
+        Returns all the items of a query as a list.
 
         Returns:
-            list: The list of query objects.
+            list: List of query items
         """
         return self._perform_query()
 
     def first(self):
         """
-        Returns the first result of this query.
+        Returns the first item that would be returned as the result of a query.
 
         Returns:
-            object: The first result of this query, or None if it has no results.
+            obj: First query item
         """
         res = self[:1]
-        if not len(res):
+        if res is None or not len(res):
             return None
         return res[0]
 
     def one(self):
         """
-        Returns the single result of this query.
+        Returns the only item that would be returned by a query.
 
         Returns:
-            object: The single result of this query.
+            obj: Sole query return item
 
         Raises:
-            MoreThanOneResultError: If the query has any number of results other than 1.
+            MoreThanOneResultError: If the query returns zero items, or more than one item
         """
         res = self[:2]
-
-        if len(res) == 0 or len(res) > 1:
-            raise MoreThanOneResultError(message="{0:d} results found for query {1:s}"
-                                         .format(len(self), str(self._query)))
-
+        if res is None:
+            return None
+        if len(res) == 0:
+            raise MoreThanOneResultError(
+                message="0 results for query {0:s}".format(self._query)
+            )
+        if len(res) > 1:
+            raise MoreThanOneResultError(
+                message="{0:d} results found for query {1:s}".format(
+                    len(self), self._query
+                )
+            )
         return res[0]
 
-    def _perform_query(self):
-        # This has the effect of generating an empty iterator.
-        yield from ()
-
     def __len__(self):
         """
-        Returns the number of items returned by this query.
+        Return the number of objects this query returns.
 
         Returns:
-            int: The number of items returned by this query.
+            int: The number of objects this query returns.
         """
-        return 0
+        return self._count()
 
     def __getitem__(self, item):
         """
-        Return a specific item or items from the query.
+        Implements list index fetching for a query
 
         Args:
             item (object): Indicates the item(s) to retrieve, either as an int or a slice.
@@ -878,7 +890,13 @@ class BaseQuery(object):
         Returns:
             object: Either an item or a list of items.
         """
-        return None
+        results = list(self)
+        if isinstance(item, slice):
+            return [results[ii] for ii in range(*item.indices(len(results)))]
+        elif isinstance(item, int):
+            return results[item]
+        else:
+            raise TypeError("Invalid argument type")
 
     def __iter__(self):
         """
@@ -890,7 +908,7 @@ class BaseQuery(object):
         return self._perform_query()
 
 
-class SimpleQuery(BaseQuery):
+class SimpleQuery(BaseQuery, IterableQueryMixin):
     """A simple query object."""
     _multiple_where_clauses_accepted = False
 
@@ -1042,7 +1060,7 @@ class SimpleQuery(BaseQuery):
         return nq
 
 
-class PaginatedQuery(BaseQuery):
+class PaginatedQuery(BaseQuery, IterableQueryMixin):
     """A query that returns objects in a paginated fashion."""
     def __init__(self, cls, cb, query=None):
         """
@@ -1410,84 +1428,71 @@ class QueryBuilderSupportMixin:
         return self
 
 
-class IterableQueryMixin:
-    """A mix-in to provide iterability to a query."""
+class CriteriaBuilderSupportMixin:
+    """A mixin that supplies wrapper methods to access the _crtieria."""
 
-    def all(self):
-        """
-        Returns all the items of a query as a list.
+    def add_criteria(self, key, newlist):
+        """Add to the criteria on this query with a custom criteria key.
 
-        Returns:
-            list: List of query items
-        """
-        return self._perform_query()
-
-    def first(self):
-        """
-        Returns the first item that would be returned as the result of a query.
-
-        Returns:
-            obj: First query item
-        """
-        allres = list(self)
-        res = allres[:1]
-        if not len(res):
-            return None
-        return res[0]
-
-    def one(self):
-        """
-        Returns the only item that would be returned by a query.
-
-        Returns:
-            obj: Sole query return item
-
-        Raises:
-            MoreThanOneResultError: If the query returns zero items, or more than one item
-        """
-        allres = list(self)
-        res = allres[:2]
-        if len(res) == 0:
-            raise MoreThanOneResultError(
-                message="0 results for query {0:s}".format(self._query)
-            )
-        if len(res) > 1:
-            raise MoreThanOneResultError(
-                message="{0:d} results found for query {1:s}".format(
-                    len(self), self._query
-                )
-            )
-        return res[0]
-
-    def __len__(self):
-        """
-        Return the number of objects this query returns.
-
-        Returns:
-            int: The number of objects this query returns.
-        """
-        return self._count()
-
-    def __getitem__(self, item):
-        """
-        Not implemented.
+        Will overwrite any existing criteria for the specified key.
 
         Args:
-            item (int): Unused.
+            key (str): The key for the criteria item to be set.
+            newlist (str or list[str]): Value or list of values to be set for the criteria item.
 
         Returns:
-            None
-        """
-        return None
+            The query object with specified custom criteria.
 
-    def __iter__(self):
+        Example:
+            query = api.select(Event).add_criteria("event_type", ["filemod", "scriptload"])
+            query = api.select(Event).add_criteria("event_type", "filemod")
         """
-        Returns the iterator for this query.
+        if not isinstance(newlist, list):
+            if not isinstance(newlist, str):
+                raise ApiError("Criteria value(s) must be a string or list of strings. "
+                               f"{newlist} is a {type(newlist)}.")
+            self._update_criteria(key, [newlist], overwrite=True)
+        else:
+            self._update_criteria(key, newlist, overwrite=True)
+        return self
+
+    def update_criteria(self, key, newlist):
+        """Update the criteria on this query with a custom criteria key.
+
+        Args:
+            key (str): The key for the criteria item to be set.
+            newlist (list): List of values to be set for the criteria item.
 
         Returns:
-            Iterator: The iterator for this query.
+            The query object with specified custom criteria.
+
+        Example:
+            query = api.select(Alert).update_criteria("my.criteria.key", ["criteria_value"])
+
+        Note: Use this method if there is no implemented method for your desired criteria.
         """
-        return self._perform_query()
+        if not isinstance(newlist, list):
+            if not isinstance(newlist, str):
+                raise ApiError("Criteria value(s) must be a string or list of strings. "
+                               f"{newlist} is a {type(newlist)}.")
+            self._update_criteria(key, [newlist])
+        else:
+            self._update_criteria(key, newlist)
+        return self
+
+    def _update_criteria(self, key, newlist, overwrite=False):
+        """
+        Updates a list of criteria being collected for a query, by setting or appending items.
+
+        Args:
+            key (str): The key for the criteria item to be set.
+            newlist (list): List of values to be set for the criteria item.
+            overwrite (bool): Overwrite the existing criteria for specified key
+        """
+        if self._criteria.get(key, None) is None or overwrite:
+            self._criteria[key] = newlist
+        else:
+            self._criteria[key].extend(newlist)
 
 
 class AsyncQueryMixin:
@@ -1525,7 +1530,268 @@ class AsyncQueryMixin:
         return self._cb._async_submit(lambda arg, kwarg: arg[0]._run_async_query(arg[1]), self, context)
 
 
-class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin):
+class Query(PaginatedQuery, QueryBuilderSupportMixin, IterableQueryMixin, AsyncQueryMixin, CriteriaBuilderSupportMixin):
+    """Represents a prepared query to the Cb Enterprise EDR backend.
+
+    This object is returned as part of a `CbEnterpriseEDRAPI.select`
+    operation on models requested from the Cb Enterprise EDR backend. You should not have to create this class yourself.
+
+    The query is not executed on the server until it's accessed, either as an iterator (where it will generate values
+    on demand as they're requested) or as a list (where it will retrieve the entire result set and save to a list).
+    You can also call the Python built-in ``len()`` on this object to retrieve the total number of items matching
+    the query.
+
+    Examples::
+
+    >>> from cbc_sdk import CBCloudAPI
+    >>> from cbc_sdk.enterprise_edr import Report
+    >>> cb = CBCloudAPI()
+    >>> query = cb.select(Report)
+    >>> query = query.where(report_id="ABCDEFG1234")
+    >>> # alternatively:
+    >>> query = query.where("report_id:ABCDEFG1234")
+
+    Notes:
+        - The slicing operator only supports start and end parameters, but not step. ``[1:-1]`` is legal, but
+          ``[1:2:-1]`` is not.
+        - You can chain where clauses together to create AND queries; only objects that match all ``where`` clauses
+          will be returned.
+    """
+
+    def __init__(self, doc_class, cb):
+        """
+        Initialize the Query object.
+
+        Args:
+            doc_class (class): The class of the model this query returns.
+            cb (CBCloudAPI): A reference to the CBCloudAPI object.
+        """
+        super(Query, self).__init__(doc_class, cb, None)
+
+        self._query_builder = QueryBuilder()
+        self._criteria = {}
+        self._exclusions = {}
+        self._sort_by = []
+        self._group_by = None
+        self._batch_size = 500
+        self._start = 0
+        self._time_range = {}
+        self._fields = ["*"]
+        self._default_args = {}
+
+    def add_exclusions(self, key, newlist):
+        """Add to the excluions on this query with a custom exclusion key.
+
+        Args:
+            key (str): The key for the exclusion item to be set.
+            newlist (str or list[str]): Value or list of values to be set for the exclusion item.
+
+        Returns:
+            The ResultQuery with specified custom exclusion.
+
+        Example:
+            query = api.select(Event).add_exclusions("netconn_domain", ["www.google.com"])
+            query = api.select(Event).add_exclusions("netconn_domain", "www.google.com")
+        """
+        if not isinstance(newlist, list):
+            if not isinstance(newlist, str):
+                raise ApiError("Exclusion value(s) must be a string or list of strings. "
+                               f"{newlist} is a {type(newlist)}.")
+            self._add_exclusions(key, [newlist])
+        else:
+            self._add_exclusions(key, newlist)
+        return self
+
+    def _add_exclusions(self, key, newlist):
+        """
+        Updates a list of exclusion being collected for a query, by setting or appending items.
+
+        Args:
+            key (str): The key for the exclusion item to be set.
+            newlist (list): List of values to be set for the exclusion item.
+        """
+        oldlist = self._exclusions.get(key, [])
+        self._exclusions[key] = oldlist + newlist
+
+    def set_fields(self, fields):
+        """
+        Sets the fields to be returned with the response.
+
+        Args:
+            fields (str or list[str]): Field or list of fields to be returned.
+        """
+        if not isinstance(fields, list):
+            if not isinstance(fields, str):
+                raise ApiError(f"Fields must be a string or list of strings. {fields} is a {type(fields)}.")
+            self._fields = [fields]
+        else:
+            self._fields = fields
+        self._default_args["fields"] = self._fields
+        return self
+
+    def set_start(self, start):
+        """
+        Sets the 'start' query body parameter, determining where to begin retrieving results from.
+
+        Args:
+            start (int): Where to start results from.
+        """
+        if not isinstance(start, int):
+            raise ApiError(f"Start must be an integer. {start} is a {type(start)}.")
+        self._start = start
+        self._default_args["start"] = self._start
+        return self
+
+    def set_rows(self, rows):
+        """
+        Sets the 'rows' query body parameter, determining how many rows of results to request.
+
+        Args:
+            rows (int): How many rows to request.
+        """
+        if not isinstance(rows, int):
+            raise ApiError(f"Rows must be an integer. {rows} is a {type(rows)}.")
+        self._batch_size = rows
+        self._default_args["rows"] = self._batch_size
+        return self
+
+    def set_time_range(self, start=None, end=None, window=None):
+        """
+        Sets the 'time_range' query body parameter, determining a time window based on 'device_timestamp'.
+
+        Args:
+            start (str in ISO 8601 timestamp): When to start the result search.
+            end (str in ISO 8601 timestamp): When to end the result search.
+            window (str): Time window to execute the result search, ending on the current time.
+                Should be in the form "-2w", where y=year, w=week, d=day, h=hour, m=minute, s=second.
+
+        Note:
+            - `window` will take precendent over `start` and `end` if provided.
+
+        Examples:
+            query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z")
+            second_query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z", end="2020-10-30T20:34:07Z")
+            third_query = api.select(Event).set_time_range(window='-3d')
+        """
+        if start:
+            if not isinstance(start, str):
+                raise ApiError(f"Start time must be a string in ISO 8601 format. {start} is a {type(start)}.")
+            self._time_range["start"] = start
+        if end:
+            if not isinstance(end, str):
+                raise ApiError(f"End time must be a string in ISO 8601 format. {end} is a {type(end)}.")
+            self._time_range["end"] = end
+        if window:
+            if not isinstance(window, str):
+                raise ApiError(f"Window must be a string. {window} is a {type(window)}.")
+            self._time_range["window"] = window
+        return self
+
+    def _get_query_parameters(self):
+        args = self._default_args.copy()
+        if self._criteria:
+            args["criteria"] = self._criteria
+        if self._exclusions:
+            args["exclusions"] = self._exclusions
+        if self._time_range:
+            args["time_range"] = self._time_range
+        args['query'] = self._query_builder._collapse()
+        if self._query_builder._process_guid is not None:
+            args["process_guid"] = self._query_builder._process_guid
+        if 'process_guid:' in args['query']:
+            q = args['query'].split('process_guid:', 1)[1].split(' ', 1)[0]
+            args["process_guid"] = q
+        return args
+
+    def sort_by(self, key, direction="ASC"):
+        """Sets the sorting behavior on a query's results.
+
+        Arguments:
+            key (str): The key in the schema to sort by.
+            direction (str): The sort order, either "ASC" or "DESC".
+
+        Returns:
+            Query: The query with sorting parameters.
+
+        Example:
+
+        >>> cb.select(Process).where(process_name="cmd.exe").sort_by("device_timestamp")
+        """
+        found = False
+
+        for sort_item in self._sort_by:
+            if sort_item['field'] == key:
+                sort_item['order'] = direction
+                found = True
+
+        if not found:
+            self._sort_by.append({'field': key, 'order': direction})
+
+        self._default_args['sort'] = self._sort_by
+
+        return self
+
+    def _count(self):
+        args = self._get_query_parameters()
+
+        log.debug("args: {}".format(str(args)))
+
+        result = self._cb.post_object(
+            self._doc_class.urlobject.format(
+                self._cb.credentials.org_key,
+                args["process_guid"]
+            ), body=args
+        ).json()
+
+        self._total_results = int(result.get('num_available', 0))
+        self._count_valid = True
+
+        return self._total_results
+
+    def _validate(self, args):
+        if not hasattr(self._doc_class, "validation_url"):
+            return
+
+        url = self._doc_class.validation_url.format(self._cb.credentials.org_key)
+
+        if args.get('query', False):
+            args['q'] = args['query']
+
+        # v2 search sort key does not work with v1 validation
+        args.pop('sort', None)
+
+        validated = self._cb.get_object(url, query_parameters=args)
+
+        if not validated.get("valid"):
+            raise ApiError("Invalid query: {}: {}".format(args, validated["invalid_message"]))
+
+    def _search(self, start=0, rows=0):
+        """
+        Execute the query, iterating over results 500 rows at a time.
+
+        Args:
+           start (int): What index to begin retrieving results from.
+           rows (int): Total number of results to be retrieved.
+                       If `start` is not specified, the default of 0 will be used.
+                       If `rows` is not specified, the query will continue until all available results have
+                       been retrieved, getting results in batches of 500.
+        """
+        raise NotImplementedError("_search() method must be implemented in subclass")
+
+    def _run_async_query(self, context):
+        """
+        Executed in the background to run an asynchronous query.
+
+        Args:
+            context (object): The context (always None in this case).
+
+        Returns:
+            Any: Result of the async query, which is then returned by the future.
+        """
+        return list(self._search())
+
+
+class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaBuilderSupportMixin):
     """Query class for asynchronous Facet API calls.
 
     These API calls return one result, and are not paginated or iterable.
@@ -1556,40 +1822,6 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin):
         self._ranges = []
         self._default_args = {}
 
-    def add_criteria(self, key, newlist):
-        """Add to the criteria on this query with a custom criteria key.
-
-        Args:
-            key (str): The key for the criteria item to be set.
-            newlist (str or list[str]): Value or list of values to be set for the criteria item.
-
-        Returns:
-            The ResultQuery with specified custom criteria.
-
-        Example:
-            query = api.select(Event).add_criteria("event_type", ["filemod", "scriptload"])
-            query = api.select(Event).add_criteria("event_type", "filemod")
-        """
-        if not isinstance(newlist, list):
-            if not isinstance(newlist, str):
-                raise ApiError("Criteria value(s) must be a string or list of strings. "
-                               f"{newlist} is a {type(newlist)}.")
-            self._add_criteria(key, [newlist])
-        else:
-            self._add_criteria(key, newlist)
-        return self
-
-    def _add_criteria(self, key, newlist):
-        """
-        Updates a list of criteria being collected for a query, by setting or appending items.
-
-        Args:
-            key (str): The key for the criteria item to be set.
-            newlist (list): List of values to be set for the criteria item.
-        """
-        oldlist = self._criteria.get(key, [])
-        self._criteria[key] = oldlist + newlist
-
     def add_exclusions(self, key, newlist):
         """Add to the excluions on this query with a custom exclusion key.
 
@@ -1739,6 +1971,7 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin):
         >>> cb.select(ProcessFacet).add_range({"bucket_size": 5, "start": 0, "end": 10, "field": "netconn_count"})
         >>> cb.select(ProcessFacet).add_range({"bucket_size": "+1DAY", "start": "2020-11-01T00:00:00Z",
                                                "end": "2020-11-12T00:00:00Z", "field": "backend_timestamp"})
+
         """
         if isinstance(range, dict):
             self._check_range(range)
@@ -1896,20 +2129,17 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin):
             query_parameters = {}
 
         result = self._cb.get_object(result_url, query_parameters=query_parameters)
-        yield self._doc_class(self._cb, model_unique_id=self._query_token, initial_data=result)
+        return self._doc_class(self._cb, model_unique_id=self._query_token, initial_data=result)
 
     def _perform_query(self):
-        for item in self.results:
-            yield item
+        return self.results
 
     @property
     def results(self):
         """Save query results to self._results with self._search() method."""
         if not self._full_init:
-            for item in self._search():
-                self._results = item
+            self._results = self._search()
             self._full_init = True
-
         return self._results
 
     def _init_async_query(self):
@@ -1932,4 +2162,4 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin):
         """
         if context != self._query_token:
             raise ApiError("Async query not properly started")
-        return list(self._search())
+        return self._search()
